//======================================================= file = mreset.c =====
//=  Reset function for REMOTE master                                         =
//=============================================================================
//=  Notes: 1)                                                                =
//=---------------------------------------------------------------------------=
//=  Contains: mreset() - Resets remote hosts                                 =
//=---------------------------------------------------------------------------=
//=  History:  KJC (09/19/01) - Genesis                                       =
//=============================================================================
#define EXTERNAL               // This is an external file (used in master.h)

//----- Include files ---------------------------------------------------------
#include "master.h"            // Needed for master stuff
#include "common.h"            // Needed for common stuff

//=============================================================================
//= Function to reset remote hosts                                            =
//=============================================================================
int mreset(char *host_name)
{
  FILE          *st;                // File stream
  char          host[80];           // Host name read from host list
  int           retcode;            // Return code

  // Test if for all hosts
  if ((strcmp(host_name, "ALL") == 0) || (strcmp(host_name, "All") == 0) ||
      (strcmp(host_name, "all") == 0))
  {
    // Open host_list
    st = fopen(HOST_LIST, "r");
    if (st == NULL)
    {
       printf("*** ERROR in mreset() - unable to open host list %s \n",
         HOST_LIST);
       return(-1);
    }

    while(TRUE)
    {
      fscanf(st, "%s", host);
      if (feof(st)) break;

      // Output banner
      printf("Sending a reset to host %s... \n", host);

      // Send reset command
      retcode = send_command("RESET", host, RESET_PORT);
      if (retcode < 0)
      {
        printf("*** ERROR in mreset() - send_command() failed \n");
        return(-1);
      }
      printf("Command send complete \n");
    }
    fclose(st);
  }
  else
  {
    // Output banner
    printf("Sending a reset to host %s... \n", host_name);

    // Send reset command
    retcode = send_command("RESET", host_name, RESET_PORT);
    if (retcode < 0)
    {
      printf("*** ERROR in mreset() - send_command() failed \n");
      return(-1);
    }
    printf("Command send complete \n");
  }

  return(1);
}
