//====================================================== file = mstatus.c =====
//=  Status function for REMOTE master                                        =
//=============================================================================
//=  Notes: 1)                                                                =
//=---------------------------------------------------------------------------=
//=  Contains: mstatus() - Outputs status datagrams to the console            =
//=---------------------------------------------------------------------------=
//=  History:  KJC (09/19/01) - Genesis                                       =
//=============================================================================
#define EXTERNAL               // This is an external file (used in master.h)

//----- Include files ---------------------------------------------------------
#include "master.h"            // Needed for master stuff
#include "common.h"            // Needed for common stuff

//=============================================================================
//= Function to output status datagrams to the console                        =
//=============================================================================
int mstatus(char *host_name)
{
  WORD wVersionRequested = MAKEWORD(1,1);  // WSA functions
  WSADATA wsaData;                         // Stuff for WSA functions
  unsigned int         master_s;           // Server socket descriptor
  struct sockaddr_in   master_addr;        // Server Internet address
  int                  addr_len;           // Internet address length
  char                 in_buf[DGRAM_SIZE]; // Buffer for input data
  int                  retcode;            // Return code

  // Output banner
  if (strcmp(host_name, "ALL") == 0)
    printf("Outputting remote status datagrams for all remotes... \n");
  else
    printf("Outputting remote status datagrams for remote %s... \n",
      host_name);
  printf("  >>> Press any key to terminate <<< \n");

  // Initializes winsock
  WSAStartup(wVersionRequested, &wsaData);

  // Create a socket, fill-in address information, and bind
  master_s = socket(AF_INET, SOCK_DGRAM, 0);
  master_addr.sin_family = AF_INET;
  master_addr.sin_port = htons(STATUS_PORT);
  master_addr.sin_addr.s_addr = htonl(INADDR_ANY);
  retcode = bind(master_s, (struct sockaddr *)&master_addr, sizeof(master_addr));
  if (retcode < 0)
  {
    printf("*** ERROR in mstatus() - bind() failed \n");
    return(-1);
  }

  // Add a blank space to the end of hostname for stopping a string search
  strcat(host_name, " ");

  // Receive and output remote status datagrams until a key is pressed
  while(TRUE)
  {
    addr_len = sizeof(master_addr);
    retcode = recvfrom(master_s, in_buf, sizeof(in_buf), 0,
                       (struct sockaddr *)&master_addr, &addr_len);
    if (retcode < 0)
    {
      printf("*** WARNING in mstatus() - recvfrom() failed (%d) \n",
      WSAGetLastError());
    }

    // Output datagram to console
    if (strcmp(host_name, "ALL ") == 0)
      printf("%s \n", in_buf);
    else if (strstr(in_buf, host_name) != NULL)
      printf("%s \n", in_buf);

    // Break if a key has been hit
    if (kbhit() > 0) break;
  }

  // Close, clean-up, and return
  closesocket(master_s);
  WSACleanup();

  return(1);
}
