//======================================================= file = remote.c =====
//=  Main program for REMOTE remote                                           =
//=   - Catches and runs simulation from a master PC                          =
//=============================================================================
//=  Notes:  1) Ignore compile time warnings on unreachable code and          =
//=             thread_arg never used.                                        =
//=---------------------------------------------------------------------------=
//=  Build:  Standard C build for a Winsock application.  For Borland         =
//=          compiler:                                                        =
//=            bcc32 remote.c rhelp.c rrun.c rzfer.c rreset.c rstatus.c       =
//=                  common.c                                                 =
//=---------------------------------------------------------------------------=
//=  History:  KJC (09/19/01) - Genesis                                       =
//=            KJC (04/11/02) - Added security is disabled message            =
//=============================================================================
#define MAIN                   // This is main (used in remote.h)

//----- Include files ---------------------------------------------------------
#include "remote.h"            // Needed for remote stuff
#include "common.h"            // Needed for common stuff

//=============================================================================
//==  Main program                                                           ==
//=============================================================================
void main(int argc, char *argv[])
{
  char         command[80];    // Command
  int          xfer_id;        // Thread id for xfer function
  int          run_id;         // Thread id for run function
  int          whois_id;       // Thread id for whois function
  int          status_id;      // Thread id for status function
  int          i;              // Loop counter
  int          retcode;        // Return code

  // Parse the command and call the appropriate function
  if (argc == 1)
    rhelp();
  else
  {
    // Convert command in argv[1] to all upper case
    for (i=0; i<((int) strlen(argv[1]) + 1); i++)
      command[i] = toupper(argv[1][i]);

    // Parse the command and any options
    if ((strcmp(command, "HELP") == 0) || (strcmp(command, "?") == 0))
      rhelp();
    else if (strcmp(command, "RUN") == 0)
    {
      if (argc == 3)
      {
        strcpy(Master_name, argv[2]);
        retcode = get_addr(Master_name, Master_ip_addr);
        if (retcode < 0)
        {
          printf("*** ERROR in main() - Cannot resolve host %s to IP addr \n",
            Master_name);
          return;
        }

        // Kick everything off and run
        while(1)
        {
          Busy_flag = FALSE;
          get_time(Time);
          printf("Starting remote for master %s at %s... \n", Master_name, Time);
          if (SEC_FLAG == FALSE) printf("*** Security is disabled *** \n");
          printf("  >>> Ctrl-C to break-out <<< \n");
          rxfer(&xfer_id);
          rrun(&run_id);
          rstatus(&status_id);
          rreset(xfer_id, run_id, status_id);
        }
      }
      else
        rhelp();
    }
    else
      rhelp();
  }
}
