//======================================================= file = rreset.c =====
//=  Reset function for REMOTE remote                                         =
//=============================================================================
//=  Notes: 1)                                                                =
//=---------------------------------------------------------------------------=
//=  Contains: rreset() - Terminates all threads and returns to main()        =
//=---------------------------------------------------------------------------=
//=  History:  KJC (09/19/01) - Genesis                                       =
//=============================================================================
#define EXTERNAL               // This is an external file (used in remote.h)

//----- Include files ---------------------------------------------------------
#include "remote.h"            // Needed for remote stuff
#include "common.h"            // Needed for common stuff

//----- Global stuff for dealing with Process ---------------------------------
EXTERN int        Proc_handle; // Process handle for simulation

//=============================================================================
//= Function to terminate all threads and return to main()                    =
//=============================================================================
int rreset(int xfer_id, int run_id, int status_id)
{
  int           arg;                // Thread arg
  char          host_name[80];      // Remote host name
  char          command[80];        // Command
  int           i;                  // Loop counter
  int           retcode;            // Return code

  // Wait for a RESET command to be received
  while(TRUE)
  {
    retcode = recv_command(command, host_name, RESET_PORT);
    if (retcode < 0)
    {
      printf("*** ERROR in rrun() - recv_command() failed \n");
      return(-1);
    }

    // Convert command to all upper case
    for (i=0; i<((int) strlen(command) + 1); i++)
      command[i] = toupper(command[i]);

    // Check if it is a RESET command
    if (strcmp(command, "RESET") == 0) break;
  }

  // Output a banner
  printf("Resetting remote... \n");

  // Terminate simulation process if BUSY
  if (Busy_flag == TRUE)
  {
    TerminateProcess((HANDLE) Proc_handle, -1);
    printf("*** NOTE - Running simulation program was terminated \n");
    Sleep(RESET_DELAY);
  }

  // Terminate all threads
  TerminateThread((HANDLE) xfer_id, (DWORD) arg);
  TerminateThread((HANDLE) run_id, (DWORD) arg);
  TerminateThread((HANDLE) status_id, (DWORD) arg);

  // Delay for closing time
  Sleep(TURN_DELAY);

  // Issue a WSACleanup() for every issued WSAStartUp that may have been issued
  for (i=0; i<10; i++)
    WSACleanup();

  // Delay for everything to wrap-up and then output a message
  Sleep(TURN_DELAY);
  printf("Reset complete \n");

  return(1);
}
