//========================================================= file = rrun.c =====
//=  Run function for REMOTE remote                                           =
//=============================================================================
//=  Notes: 1)                                                                =
//=---------------------------------------------------------------------------=
//=  Contains: rrun() - Runs programs sent from master                        =
//=---------------------------------------------------------------------------=
//=  History:  KJC (09/19/01) - Genesis                                       =
//=============================================================================
#define EXTERNAL               // This is an external file (used in remote.h)

//----- Include files ---------------------------------------------------------
#include "remote.h"            // Needed for remote stuff
#include "common.h"            // Needed for common stuff

//----- Global stuff for dealing with Process ---------------------------------
PROCESS_INFORMATION ProcInfo;    // Structure for PROCESS_INFORMATION
STARTUPINFO         StartUpInfo; // Structure for STARTUPINFO
DWORD               dwResult;    // Double world result variable
HANDLE              Proc_handle; // Process handle for simulation

//=============================================================================
//= Function to run programs sent from master                                 =
//=============================================================================
int rrun(int *run_id)
{
  // Kick-off thread
  *run_id = _beginthread(rrun_thread, 4096, NULL);
  if (*run_id == -1)
  {
    printf("*** ERROR - Unable to create thread for rrun() \n");
    return(-1);
  }

  return(1);
}

//===========================================================================
//=  Thread code for rrun()                                                 =
//===========================================================================
void rrun_thread(void *thread_arg)
{
  FILE          *fp;                // Pointer to status output file
  char          host_name[80];             // Remote host name
  char          exe_file[80];       // Executable file name
  char          in_file[80];        // Input file name
  char          out_file[80];       // Output file name
  char          buf[80];            // Character buffer
  int           retcode;            // Return code

  while(TRUE)
  {
    // Receive the executable file
    retcode = recv_file(exe_file, host_name, RUN_PORT1);
    if (retcode < 0)
    {
      printf("*** ERROR in rrun_thread() - recv_file() failed \n");
      _endthread();
    }

    // Receive the input file
    retcode = recv_file(in_file, host_name, RUN_PORT1);
    if (retcode < 0)
    {
      printf("*** ERROR in rrun_thread() - recv_file() failed \n");
      _endthread();
    }

    // Receive the output file
    retcode = recv_file(out_file, host_name, RUN_PORT1);
    if (retcode < 0)
    {
      printf("*** ERROR in rrun_thread() - recv_file() failed \n");
      _endthread();
    }

    // Open status.txt for writing status updates
    fp = fopen("status.txt", "w");
    if (fp == NULL)
    {
      printf("*** ERROR in opening status file 'status.txt' \n");
      _endthread();
    }
    fprintf(fp, "(%s %s %s) - ", exe_file, in_file, out_file);
    fclose(fp);

    // Security check that this remote execution is from known Master
    if (SEC_FLAG == TRUE)
    {
      if ((strcmp(host_name, Master_name) != 0) &&
          (strcmp(host_name, "localhost") != 0))
      {
        printf("*** ERROR in rrun_thread() - security check failed (%s) \n",
          host_name);
        _endthread();
      }
    }

    // Check if already busy, if so we have a problem
    if (Busy_flag == TRUE)
    {
      printf("*** ERROR in rrun_thread() - already busy running a program \n");
      _endthread();
    }

    // Initialize STARTUPINFO structure
    StartUpInfo.cb = sizeof(STARTUPINFO);

    // Set-up the execution buf
    strcpy(buf, exe_file);
    strcat(buf, " ");
    strcat(buf, in_file);
    strcat(buf, " ");
    strcat(buf, out_file);

    // Create the process
    Busy_flag = TRUE;
    printf("-------------------------------------------------------------------------------\n");
    printf("Start (%s %s %s) at %s... \n", exe_file, in_file, out_file, Time);
    if (!CreateProcess(NULL, buf, NULL, NULL, FALSE,
                       CREATE_DEFAULT_ERROR_MODE, NULL, NULL,
                       &StartUpInfo, &ProcInfo))
    {
      printf("*** ERROR in rrun() - Unable to create process %s \n", buf);
      return;
    }

    // Give process time to kick-off
    Sleep(TURN_DELAY);

    // Get the process handle and wait for it to complete
    Proc_handle = OpenProcess(PROCESS_ALL_ACCESS, FALSE, ProcInfo.dwProcessId);
    WaitForSingleObject((HANDLE) Proc_handle, -1);
    get_time(Time);
    printf("Complete at %s \n", Time);
    printf("-------------------------------------------------------------------------------\n");
    Busy_flag = FALSE;

    // Send output file
    retcode = send_file(out_file, Master_name, RUN_PORT2);
    if (retcode < 0)
    {
      printf("*** ERROR in rrun_thread() - send_file() failed \n");
      _endthread();
    }
  }

  _endthread();
}
