//======================================================= file = rstatus.c ====
//=  Status function for REMOTE remote                                        =
//=============================================================================
//=  Notes: 1)                                                                =
//=---------------------------------------------------------------------------=
//=  Contains: rstatus() - Sends status datagrams to the master                =
//=---------------------------------------------------------------------------=
//=  History:  KJC (09/19/01) - Genesis                                       =
//=            KJC (10/14/01) - On fail sendto() does not return, trys again  =
//=============================================================================
#define EXTERNAL        // This is an external file (used in remote.h)

//----- Include files ---------------------------------------------------------
#include "common.h"             // Needed for common stuff
#include "remote.h"             // Needed for remote stuff

//=============================================================================
//= Function to send status datagrams to the master                           =
//=============================================================================
int rstatus(int *status_id)
{
  // Kick-off thread
  *status_id = _beginthread(rstatus_thread, 4096, NULL);
  if (*status_id == -1)
  {
    printf("*** ERROR in rstatus() - Unable to create thread \n");
    return(-1);
  }

  return(1);
}

//===========================================================================
//=  Thread code for rstatus()                                              =
//===========================================================================
void rstatus_thread(void *thread_arg)
{
  WORD wVersionRequested = MAKEWORD(1,1);       // Stuff for WSA functions
  WSADATA wsaData;                              // Stuff for WSA functions
  FILE                 *fp;                     // Pointer to status output file
  char                 status_line[BUF_SIZE];   // Status line
  int                  master_s;                // Master socket descriptor
  struct sockaddr_in   master_addr;             // Master Internet address
  int                  addr_len;                // Internet address length
  char                 my_host_name[BUF_SIZE];  // My host name
  char                 out_buf[DGRAM_SIZE];     // Buffer for output data
  int                  retcode;                 // Return code

  // Initialize winsock, create a datagram socket, and bind it
  WSAStartup(wVersionRequested, &wsaData);
  master_s = socket(AF_INET, SOCK_DGRAM, 0);
  master_addr.sin_family = AF_INET;
  master_addr.sin_port = htons(STATUS_PORT);
  master_addr.sin_addr.s_addr = inet_addr(Master_ip_addr);

  // Get my hostname
  gethostname(my_host_name, sizeof(my_host_name));

  // Open status.txt for reading status updates
  //  - If cannot open an existing "status.txt" then create a "status.txt" file
  fp = fopen("status.txt", "r");
  if (fp == NULL)
  {
    fp = fopen("status.txt", "w+");
    if (fp == NULL)
    {
      printf("*** ERROR in opening status file 'status.txt' \n");
      exit(1);
    }
  }

  // Loop to send messages
  while(TRUE)
  {
    // Read current status line
    rewind(fp);
    fgets(status_line, BUF_SIZE, fp);

    // Create message to be sent
    strcpy(out_buf, my_host_name);
    if (Busy_flag == TRUE)
    {
      strcat(out_buf, " - ");
      strcat(out_buf, status_line);
    }
    else
      strcat(out_buf, " - Idle");

    // Send the message
    retcode = sendto(master_s, out_buf, (strlen(out_buf) + 1), 0,
      (struct sockaddr *)&master_addr, sizeof(master_addr));
    if (retcode < 0)
    {
      printf("*** WARNING in rstatus_thread() - sendto() failed (%d) \n",
      WSAGetLastError());
    }

    // Delay for STATUS_DELAY seconds
    Sleep(STATUS_DELAY);
  }

  // Close and clean-up
  closesocket(master_s);
  WSACleanup();

  _endthread();
}
